#!/usr/bin/env node
"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("@babel/polyfill");

var _path = _interopRequireDefault(require("path"));

var _commander = _interopRequireDefault(require("commander"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _contract = require("./contract");

var _wallet = require("./wallet");

var _cliUtils = require("./cli-utils");

/*
Using babel polyfill until ledger modules are updated:
https://github.com/LedgerHQ/ledgerjs/issues/211
*/
_commander.default.version('0.1.0').option('-c, --config-path <path>', 'Set path to config file').option('-a, --abi-dir <path>', 'Set path to ABI folder').option('-r, --rpc-url <url>', 'Set ethereum JSON RPC URL').option('-t, --testnet', 'Flag to use testnet').parse(process.argv);

const ethNetwork = _commander.default.testnet ? 'testnet' : 'mainnet'; // testnet or mainnet

let config = {
  rpcUrl: {},
  abiDirPath: './abi'
};

if (_commander.default.configPath) {
  const configPath = _path.default.resolve(process.cwd(), _commander.default.configPath);

  config = _fsExtra.default.readJsonSync(configPath);
}

if (_commander.default.rpcUrl) {
  config.rpcUrl = {
    [ethNetwork]: _commander.default.rpcUrl
  };
}

if (_commander.default.abiDir) {
  config.abiDirPath = _path.default.resolve(process.cwd(), _commander.default.abiDir);
} else {
  _cliUtils.log.warn('No abi directory path provided');

  _cliUtils.log.info(`please prpovide a relative path to the folder containing your contract ABI's, e.g. --abi-dir=./abi`);

  process.exit();
}

if (!Object.keys(config.rpcUrl).length) {
  _cliUtils.log.warn('rpc url required, please set config file or pass as arg, e.g.: --rpc-url="https://ropsten.infura.io/FWLG9Y..."');

  process.exit();
}

_cliUtils.log.header(`\nEthereum ${ethNetwork}\n`);

const startCli = async () => {
  try {
    const {
      web3,
      account
    } = await (0, _wallet.openWallet)(ethNetwork, config.rpcUrl[ethNetwork]);

    _cliUtils.log.info(`Using account: ${account}`, 'blue');

    await (0, _contract.callContract)(web3, ethNetwork, config);
  } catch (err) {
    if (err.message && err.message.includes('Failed to subscribe to new newBlockHeaders')) {
      _cliUtils.log.info('Please visit etherscan to check transaction status for transactions signed by a ledger');
    } else {
      _cliUtils.log.warn('Error: ' + err.message);

      _cliUtils.log.info(err.stack);

      process.exit();
    }
  }
};

startCli();