"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.openWallet = void 0;

require("@babel/polyfill");

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _web3EthAccounts = _interopRequireDefault(require("web3-eth-accounts"));

var _web = _interopRequireDefault(require("web3"));

var _truffleHdwalletProvider = _interopRequireDefault(require("truffle-hdwallet-provider"));

var _truffleHdwalletProviderPrivkey = _interopRequireDefault(require("truffle-hdwallet-provider-privkey"));

var _web3ProviderEngine = _interopRequireDefault(require("web3-provider-engine"));

var _rpc = _interopRequireDefault(require("web3-provider-engine/subproviders/rpc"));

var _hwTransportNodeHid = _interopRequireDefault(require("@ledgerhq/hw-transport-node-hid"));

var _web3Subprovider = _interopRequireDefault(require("@ledgerhq/web3-subprovider"));

var _inquirer = require("inquirer");

var _cliUtils = require("./cli-utils");

/*
Using babel polyfill until ledger modules are updated:
https://github.com/LedgerHQ/ledgerjs/issues/211
*/
const decodeKeystore = async path => {
  const questions = [{
    type: 'input',
    name: 'password',
    message: 'Enter password to decrypt file: '
  }];

  if (!path) {
    questions.unshift({
      type: 'input',
      name: 'keystorePath',
      message: 'Enter path of keystore file: '
    });
  }

  const answers = await (0, _inquirer.prompt)(questions);
  const file = await _fsExtra.default.readJson(path || answers.keystorePath);
  const accounts = new _web3EthAccounts.default();
  const decrypted = accounts.decrypt(file, answers.password);
  return decrypted.privateKey;
};

const requestPrivateKey = async () => {
  const answers = await (0, _inquirer.prompt)([{
    type: 'password',
    name: 'privateKey',
    message: 'Enter private key: '
  }]);
  return answers.privateKey;
};

const requestMnemonic = async () => {
  const answers = await (0, _inquirer.prompt)([{
    type: 'password',
    name: 'mnemonic',
    message: 'Enter mnemonic passphrase: '
  }]);
  return answers.mnemonic;
};

const requestAccountSelection = async accounts => {
  const answers = await (0, _inquirer.prompt)([{
    type: 'list',
    name: 'account',
    message: 'Select ETH account: ',
    choices: accounts
  }]);
  return accounts.indexOf(answers.account);
};

const openWallet = async (ethNetwork, rpcUrl) => {
  const answers = await (0, _inquirer.prompt)([{
    type: 'list',
    name: 'wallet',
    message: 'Wallet source:',
    choices: ['Private Key', 'Mnemonic', 'Ledger', 'Keystore File']
  }]);

  if (answers.wallet === 'Ledger') {
    const engine = new _web3ProviderEngine.default();
    const web3 = new _web.default(engine);

    try {
      const ledgerIndexAnswers = await (0, _inquirer.prompt)([{
        type: 'input',
        name: 'accountIndex',
        default: 0,
        message: 'Ledger accounts selection offset: '
      }]);
      const networkId = ethNetwork === 'testnet' ? 3 : 1;

      const getTransport = () => _hwTransportNodeHid.default.create();

      const ledgerWalletSelectedProvider = (0, _web3Subprovider.default)( //$FlowFixMe Transport type
      getTransport, {
        networkId,
        accountsLength: 5,
        accountsOffset: Number(ledgerIndexAnswers.accountIndex)
      });
      engine.addProvider(ledgerWalletSelectedProvider);
      engine.addProvider(new _rpc.default({
        rpcUrl
      }));
      engine.start();
      const accounts = await web3.eth.getAccounts();
      const accountIndex = await requestAccountSelection(accounts);
      const account = accounts[accountIndex];
      return {
        web3,
        account
      };
    } catch (err) {
      _cliUtils.log.warn(err);

      _cliUtils.log.info(`Error connecting to Ledger, please make sure:\n
      - You have entered your pin and unlocked your ledger\n
      - Selected Ethereum wallet\n
      - Have browser support turned off`);

      throw err;
    }
  } else {
    let provider;

    switch (answers.wallet) {
      case 'Keystore File':
        {
          let decodedKey = await decodeKeystore();

          if (decodedKey.slice(0, 2) === '0x') {
            decodedKey = decodedKey.slice(2, decodedKey.length);
          }

          provider = new _truffleHdwalletProviderPrivkey.default(decodedKey, rpcUrl);
          break;
        }

      case 'Mnemonic':
        {
          const mnemonic = await requestMnemonic();
          provider = new _truffleHdwalletProvider.default(mnemonic, rpcUrl);
          break;
        }

      case 'Private Key':
      default:
        {
          const key = await requestPrivateKey();
          provider = new _truffleHdwalletProviderPrivkey.default(key, rpcUrl);
          break;
        }
    }

    const web3 = new _web.default(provider);
    const account = await web3.eth.getCoinbase();
    return {
      web3,
      account
    };
  }
};

exports.openWallet = openWallet;